/**********************************************************************
*	FILE:	ui.c
*	AUTHOR:	CALeague
*	DATE:	30 April 1993
*	PURPOSE:
*	  Houses the user interface
***********************************************************************/

#include "ui.h"

#include <stdio.h>
#include <dos.h>
#include <assert.h>
#include "errwin.h"
#include "msgwin.h"
#include "rules.h"

const int NLines = 5; 	// Max lines of text in UI message windows


void UserInterface::InitScreen (void)
{
  EraseScreen ();

  mapwin  = new MapWindow;
  keywin  = new KeyWindow;
  cpanel  = new CPanelWindow;
  playwin = new PlayerWindow;
  infowin = new InfoWindow;

  theDisplay.Add (mapwin, 0,   0);
  theDisplay.Add (keywin,  223, 403);
  theDisplay.Add (cpanel, 300, 403);
  theDisplay.Add (playwin, 0,   403);
  theDisplay.Add (infowin, 502, 403);
  theDisplay.SetLineStyle (SOLID_LINE, HORIZ_DIR, 2);
  theDisplay.SetColor (BLUE);
  theDisplay.Line (0, 401, 639, 401);
  DrawScreen ();
}

void UserInterface::ShowAllPlayers (void)
{
  for (int i=1; i<NumPlayers; i++)
  {
    Player* p = thePlayers [i];
    mapwin->HilightNode (p->Position(), p->GetColor());
  }
}


char* AwayNow [] = {"We go away now.", ""};
char* SpyMove [] = {"Spy is about to move.",
		    "Promise not to look at my Bum!?", "" };

Move UserInterface::GetPlayerMove (Player& p, MoveList& ml)
{
  if (p.IsSpy())
  {
    char buf [40];
    sprintf (buf, "Round %d", theRules.CurrentTurn());
    MessageWin (BLUE, WHITE, buf, WHITE, SpyMove, BLACK);
  }
  FlashNodes (p, ml);

  Message m;
  bool Done = False;
  int token;

  Mouse.Flush ();
  do {
    m = theDisplay.Dispatch (Mouse.Event ());
    if ((m.origin == ButtonMsg) && (m.code == ButtonPress))
      switch (m.d1)
      {
	case ID_REFRESH:
	  Refresh ();
	case ID_LEGAL:
	  FlashNodes (p, ml);
	  break;
	case ID_SPY:
	  ShowSpyMoves (thePlayers[0]->GetHistory());
	  break;
	case ID_QUIT:
	  MessageWin (RED, WHITE, "Quit Game", WHITE, AwayNow, BLACK);
	  exit (1);
	  break;
      }
    if ((m.origin == ButtonMsg) && (m.code == Status))
    {
      byte bv = ComputePossTokens (ml, m.d1);
      token = TransPop (bv, p.GetColor());
      if (token == -1)		// if they cancel
	mapwin->UnHilightNode (m.d1);
      else
	Done = True;
    }
  } while (! Done);

  mapwin->UnHilightNode (p.Position());
  mapwin->UnHilightNode (m.d1);
  mapwin->DisableNodes (ml);
  return Move (p.Position(), m.d1, On<<token);
}


void UserInterface::FlashNodes (Player& p, MoveList& ml)
{
  const int delaytime = 250;
  const int iterations = 2;

  for (int i=0; i<iterations; i++)
  {
    mapwin->EnableNodes (ml, p.GetColor());
    mapwin->HilightNode (p.Position(), p.GetColor());
    delay (delaytime);
    mapwin->DisableNodes (ml);
    mapwin->UnHilightNode (p.Position());
    delay (delaytime);
  }
  mapwin->EnableNodes (ml, p.GetColor());
  mapwin->HilightNode (p.Position(), p.GetColor());
}


int UserInterface::ComputePossTokens (MoveList& ml, int dest)
{
  int transtypes = 0;

  if (ml.Reset ())
    do {
      if (ml.GetCurrent()->Node == dest)
	transtypes = ml.GetCurrent()->Trans;
    } while (ml.Next ());

  return transtypes;
}


void UserInterface::BeginTurn (Player& p)
{
  playwin->Update (p, theRules.CurrentTurn());
  mapwin->UnHilightNode (p.Position());
}


void UserInterface::EndTurn (Player& p, Move m)
{
  mapwin->UnHilightNode (p.Position());
  mapwin->HilightNode (m.Node, p.GetColor ());
}


void UserInterface::RevealSpy (Player& p, Move m)
{
  const int nlines = 3;
  char *Text [nlines];

  for (int i=0; i<nlines; i++)
    Text [i] = new char [40];

  sprintf (Text[0], "%s was seen taking %s",
	   p.GetName(), (m.Trans==SubwayMask)? "the Subway" :
			((m.Trans==BusMask)? "the Bus" : "a Taxi"));
  sprintf (Text[1], "He will now be revealed!");
  sprintf (Text[2], "");

  MessageWin (BLUE, WHITE, "Tip", WHITE, Text, BLACK);

  MoveList ml;
  FlashNodes (p, ml);
}


void UserInterface::SpyTurn (Move& mv, int RoundN)
{
  char* Text [NLines];
  char t [40];

  for (int i=0; i<NLines; i++)
    Text [i] = new char [40];

  sprintf (t, "Tip: Round %d", RoundN+1);
  sprintf (Text [0], "Sources tell you the");
  sprintf (Text [1], "Spy was seen taking");
  if (mv.Trans & SubwayMask)
    sprintf (Text [2], "the Subway");
  else if (mv.Trans & BusMask)
    sprintf (Text [2], "the Bus");
  else
    sprintf (Text [2], "a Taxi");
  sprintf (Text [3], "");

  MessageWin (BLUE, WHITE, t, WHITE, Text, BLACK);
}


Move UserInterface::NoLegalMoves (Player& p)
{
  char* Text [5];
  for (int i=0; i<5; i++)
  {
    Text [i] = new char [40];
    assert (Text [i] != NULL);
  }
  sprintf (Text[0], "%s has", p.GetName());
  sprintf (Text[1], "no legal moves");
  sprintf (Text[2], "this turn.");
  sprintf (Text[3], "");
  MessageWin (BLUE, WHITE, "Sorry", WHITE, Text, BLACK);
  for (i=0; i<5; i++)
    delete Text [i];
  return NullMove;
}


void UserInterface::AnnounceWin (Player& P, int RoundN)
{
  char* Text [NLines];

  for (int i=0; i<NLines; i++)
    Text [i] = new char [40];

  if (P.IsSpy ())
  {
    sprintf (Text [0], "** Congratulations **");
    sprintf (Text [1], "The elusive %s", P.GetName ());
    sprintf (Text [2], "escaped from the Detectives...");
    sprintf (Text [3], "");
    MoveList m;
    FlashNodes (P, m);
    FlashNodes (P, m);
  }
  else
  {
    sprintf (Text [0], "** Congratulations **");
    sprintf (Text [1], "%s", P.GetName ());
    sprintf (Text [2], "caught the spy");
    sprintf (Text [3], "in round %d.", RoundN+1);
    sprintf (Text [4], "");
  }
  MessageWin (RED, WHITE, "Game Over...", WHITE, Text, BLACK);

  for (i=0; i<NLines; i++)
    delete Text [i];
}


void UserInterface::DrawTitleScreen()
{
  theDisplay.SetTextJustify (CENTER_TEXT, CENTER_TEXT);
  theDisplay.SetTextStyle (TRIPLEX_FONT, 0, 8);
  theDisplay.OutTextXY (theDisplay.GetMaxX()/2, 35, "Scotland Yard");
}

