/*************************************************************************
* 	FILE:	window.c
*	AUTHOR: CALeague
*	DATE:	25 April 1993
*	PURPOSE:
*	  A window is a rectangular screen object which has a border and
*	  may have a title bar.  The area contained inside the border and
*	  title bar is called the client area.  The client area of a window
*	  is, itself, a Display.  This means that you can draw on it
*	  and it can contain ScreenObjects, such as buttons and windows.
**************************************************************************/

#include "window.h"
#include <alloc.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>


/**********************************************************************
*	Constructors
***********************************************************************/

Window::Window (void)		// Dangerous constructor!
{				// Only to be used by derived types.
  XDim = YDim = 0;
  CliX1 = CliY1 = CliX2 = CliY2 = 0;
  TX1 = TY1 = TX2 = TY2 = 0;
  BorderColor = BkColor = 0;
  TitleColor = 0;
  Title = NULL;
  ClientD = TitleD = NULL;
  Drawn = False;
}

Window::Window (int dx, int dy, int bc, int bkc)
{
  XDim = dx;  YDim = dy;
  CliX1 = CliY1 = CliX2 = CliY2 = 0;
  TX1 = TY1 = TX2 = TY2 = 0;
  BorderColor = bc;
  BkColor = bkc;
  TitleColor = 0;
  Title = NULL;
  ClientD = TitleD = NULL;
  Drawn = False;
}

Window::Window (int dx, int dy, int bc, int bkc, char* t, int tc)
{
  XDim = dx;  YDim = dy;
  CliX1 = CliY1 = CliX2 = CliY2 = 0;
  TX1 = TY1 = TX2 = TY2 = 0;
  BorderColor = bc;
  BkColor = bkc;
  TitleColor = tc;
  Title = new char [strlen (t)+1];
  assert (Title != 0);
  strcpy (Title, t);
  ClientD = TitleD = NULL;
  Drawn = False;
}


/**********************************************************************
*	Destructor
***********************************************************************/

Window::~Window ()
{
  Erase ();
  delete Title;
  delete ClientD;
  delete TitleD;
}


/**********************************************************************
*	Calculations
***********************************************************************/

void Window::CalcBorder (void)
{
  CliX1 += X + BorderWidth+1;
  CliY1 += Y + BorderWidth+1;
  CliX2 += X + XDim - BorderWidth - 1;
  CliY2 += Y + YDim - BorderWidth - 1;
}

void Window::CalcTitle (void)
{
  TX1 += CliX1;
  TY1 += CliY1;
  TX2 += CliX2;
  TY2 += TY1 + TitleWidth;
  CliY1 += TitleWidth + 2;
}


/**********************************************************************
*	Init
***********************************************************************/

void Window::Init (Display *d, int x, int y)
{
  Disp = d;
  X = x;  Y = y;

  CalcBorder ();
  if (Title)
  {
    CalcTitle ();
    TitleD = new Display (Disp, TX1, TY1, TX2, TY2);
    assert (TitleD != 0);
    TitleD->SetBkColor (BorderColor);
    TitleD->SetTextJustify (CENTER_TEXT, TOP_TEXT);
    TitleD->SetTextStyle (TitleFont, HORIZ_DIR, TitleFontSize);
    TitleD->SetColor (TitleColor);
  }
  ClientD = new Display (Disp, CliX1, CliY1, CliX2, CliY2);
  assert (ClientD != 0);
  ClientD->SetBkColor (BkColor);
}


/**********************************************************************
*	Drawing
***********************************************************************/

void Window::DrawBorder (void)
{
  Disp->SetLineStyle (SOLID_LINE, 1, 1);
  Disp->SetColor (BorderColor);
  for (int i=0; i<BorderWidth; i++)
    Disp->Rectangle (X+i, Y+i, X+XDim-i, Y+YDim-i);
  Disp->SetColor (BLACK);
  Disp->Rectangle (X+BorderWidth, Y+BorderWidth,
		   X+XDim-BorderWidth, Y+YDim-BorderWidth);
}


void Window::DrawTitle (void)
{
  Disp->SetColor (BLACK);
  Disp->Line (TX1, TY2+1, TX2, TY2+1);
  TitleD->Clear ();
  TitleD->OutTextXY (TitleD->GetMaxX()/2, 0, Title);
}


void Window::Draw (void)
{
  if (!Drawn)
  {
    assert (Disp != NULL);
    Disp->FastDraw ();
    PrepScreen ();
    DrawBorder ();
    if (TitleD)
      DrawTitle ();
    ClientD->Clear ();
    ClientD->Draw ();
    Disp->Done ();
    Drawn = True;
  }
}


/**********************************************************************
*	Erase
***********************************************************************/

void Window::Erase (void)
{
  if (Drawn)
  {
    assert (Disp != NULL);
    ClientD->Erase ();
    Disp->SetFillStyle (SOLID_FILL, Disp->GetBkColor());
    Disp->Bar (X, Y, X+XDim, Y+YDim);
    Drawn = False;
  }
}


/**********************************************************************
*	Dispatch
***********************************************************************/

Message Window::Dispatch (Message& msg)
{
  if (ClientD)
    return ClientD->Dispatch (msg);	// Dispatch messages to client area
  else
    return NullMsg;
}

